"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeConfig = exports.validateConfig = exports.migrateUserConfig = exports.createBackup = exports.resolveConfigPathByNetworkID = exports.getConfig = exports.getLogLevel = exports.getNetworkByNetworkID = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs-extra"));
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const fs_1 = require("fs");
const path_1 = require("path");
const lisk_validator_1 = require("@liskhq/lisk-validator");
const lisk_framework_1 = require("lisk-framework");
const lisk_utils_1 = require("@liskhq/lisk-utils");
const constants_1 = require("../constants");
const path_2 = require("./path");
const LOG_LEVEL_PRIORITY = Object.freeze({
    FATAL: 0,
    ERROR: 1,
    WARN: 2,
    INFO: 3,
    DEBUG: 4,
    TRACE: 5,
});
const getNetworkByNetworkID = (_networkID) => {
    const networkInfo = constants_1.NETWORK_CONSTANT[_networkID];
    if (!networkInfo) {
        throw new Error('Migrator running against unidentified network. Cannot proceed.');
    }
    return networkInfo.name;
};
exports.getNetworkByNetworkID = getNetworkByNetworkID;
const getLogLevel = (loggerConfig) => {
    var _a, _b;
    const highestLogPriority = Math.max((_a = LOG_LEVEL_PRIORITY[String(loggerConfig.fileLogLevel || '').toUpperCase()]) !== null && _a !== void 0 ? _a : LOG_LEVEL_PRIORITY.INFO, (_b = LOG_LEVEL_PRIORITY[String(loggerConfig.consoleLogLevel || '').toUpperCase()]) !== null && _b !== void 0 ? _b : LOG_LEVEL_PRIORITY.INFO);
    try {
        const [logLevel] = Object.entries(LOG_LEVEL_PRIORITY).find(([, v]) => v === highestLogPriority);
        return logLevel.toLowerCase();
    }
    catch (err) {
        return 'info';
    }
};
exports.getLogLevel = getLogLevel;
const getConfig = async (_this, corePath, _networkID, customConfigPath) => {
    let network = 'mainnet';
    try {
        network = (0, exports.getNetworkByNetworkID)(_networkID);
    }
    catch (err) {
        _this.error(err);
    }
    const dataDirConfigPath = (0, path_1.join)(corePath, 'config', network, 'config.json');
    const dataDirConfig = await fs.readJSON(dataDirConfigPath);
    const customConfig = customConfigPath
        ? await fs.readJSON((0, path_2.resolveAbsolutePath)(customConfigPath))
        : {};
    cli_ux_1.default.action.start('Compiling Lisk Core configuration');
    const config = lisk_utils_1.objects.mergeDeep({}, dataDirConfig, customConfig);
    cli_ux_1.default.action.stop();
    return config;
};
exports.getConfig = getConfig;
const resolveConfigPathByNetworkID = async (networkIdentifier) => {
    const network = constants_1.NETWORK_CONSTANT[networkIdentifier].name;
    const configFilePath = (0, path_1.join)(__dirname, '../..', `config/${network}/config.json`);
    return configFilePath;
};
exports.resolveConfigPathByNetworkID = resolveConfigPathByNetworkID;
const createBackup = async (config) => {
    const backupPath = (0, path_1.join)(__dirname, '../..', 'backup');
    (0, fs_1.mkdirSync)(backupPath, { recursive: true });
    (0, fs_1.writeFileSync)((0, path_1.resolve)(`${backupPath}/config.json`), JSON.stringify(config, null, '\t'));
};
exports.createBackup = createBackup;
const migrateUserConfig = async (configV3, configV4, snapshotHeight) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    cli_ux_1.default.action.start('Starting migration of custom config properties.');
    if (!((_a = configV4.system) === null || _a === void 0 ? void 0 : _a.version)) {
        cli_ux_1.default.action.start(`Setting config property 'system.version' to: ${constants_1.DEFAULT_VERSION}.`);
        configV4.system.version = constants_1.DEFAULT_VERSION;
        cli_ux_1.default.action.stop();
    }
    if (configV3 === null || configV3 === void 0 ? void 0 : configV3.rootPath) {
        cli_ux_1.default.action.start(`Setting config property 'system.dataPath' to: ${configV3.rootPath}.`);
        configV4.system.dataPath = configV3.rootPath;
        cli_ux_1.default.action.stop();
    }
    if (configV3 === null || configV3 === void 0 ? void 0 : configV3.logger) {
        const logLevel = (0, exports.getLogLevel)(configV3.logger);
        cli_ux_1.default.action.start(`Setting config property 'system.logLevel' to: ${logLevel}.`);
        configV4.system.logLevel = logLevel;
        cli_ux_1.default.action.stop();
    }
    if (configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) {
        if ((_b = configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) === null || _b === void 0 ? void 0 : _b.maxTransactions) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.maxTransactions' to: ${configV3.transactionPool.maxTransactions}.`);
            configV4.transactionPool
                .maxTransactions = configV3.transactionPool.maxTransactions;
            cli_ux_1.default.action.stop();
        }
        if ((_c = configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) === null || _c === void 0 ? void 0 : _c.maxTransactionsPerAccount) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.maxTransactionsPerAccount' to: ${configV3.transactionPool.maxTransactionsPerAccount}.`);
            configV4.transactionPool
                .maxTransactionsPerAccount = configV3.transactionPool.maxTransactionsPerAccount;
            cli_ux_1.default.action.stop();
        }
        if ((_d = configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) === null || _d === void 0 ? void 0 : _d.transactionExpiryTime) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.transactionExpiryTime' to: ${configV3.transactionPool.transactionExpiryTime}.`);
            configV4.transactionPool
                .transactionExpiryTime = configV3.transactionPool.transactionExpiryTime;
            cli_ux_1.default.action.stop();
        }
        if ((_e = configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) === null || _e === void 0 ? void 0 : _e.minEntranceFeePriority) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.minEntranceFeePriority' to: ${configV3.transactionPool.minEntranceFeePriority}.`);
            configV4.transactionPool
                .minEntranceFeePriority = configV3.transactionPool.minEntranceFeePriority;
            cli_ux_1.default.action.stop();
        }
        if ((_f = configV3 === null || configV3 === void 0 ? void 0 : configV3.transactionPool) === null || _f === void 0 ? void 0 : _f.minReplacementFeeDifference) {
            cli_ux_1.default.action.start(`Setting config property 'transactionPool.minReplacementFeeDifference' to: ${configV3.transactionPool.minReplacementFeeDifference}.`);
            configV4.transactionPool
                .minReplacementFeeDifference = configV3.transactionPool.minReplacementFeeDifference;
            cli_ux_1.default.action.stop();
        }
    }
    if ((_g = configV3 === null || configV3 === void 0 ? void 0 : configV3.rpc) === null || _g === void 0 ? void 0 : _g.mode) {
        cli_ux_1.default.action.start(`Setting config property 'rpc.modes' to: ${configV3.rpc.mode}.`);
        configV4.rpc.modes = [configV3.rpc.mode];
        cli_ux_1.default.action.stop();
    }
    if (configV3 === null || configV3 === void 0 ? void 0 : configV3.network) {
        if ((_h = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _h === void 0 ? void 0 : _h.port) {
            cli_ux_1.default.action.start(`Setting config property 'network.port' to: ${configV3.network.port}.`);
            configV4.network.port = configV3.network.port;
            cli_ux_1.default.action.stop();
        }
        if ((_j = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _j === void 0 ? void 0 : _j.hostIp) {
            cli_ux_1.default.action.start(`Setting config property 'network.host' to: ${configV3.network.hostIp}.`);
            configV4.network.host = configV3.network.hostIp;
            cli_ux_1.default.action.stop();
        }
        if ((_k = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _k === void 0 ? void 0 : _k.maxOutboundConnections) {
            cli_ux_1.default.action.start(`Setting config property 'network.maxOutboundConnections' to: ${configV3.network.maxOutboundConnections}.`);
            configV4.network.maxOutboundConnections = configV3.network.maxOutboundConnections;
            cli_ux_1.default.action.stop();
        }
        if ((_l = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _l === void 0 ? void 0 : _l.maxInboundConnections) {
            cli_ux_1.default.action.start(`Setting config property 'network.maxInboundConnections' to: ${configV3.network.maxInboundConnections}.`);
            configV4.network.maxInboundConnections = configV3.network.maxInboundConnections;
            cli_ux_1.default.action.stop();
        }
        if ((_m = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _m === void 0 ? void 0 : _m.wsMaxPayload) {
            cli_ux_1.default.action.start(`Setting config property 'network.wsMaxPayload' to: ${configV3.network.wsMaxPayload}.`);
            configV4.network.wsMaxPayload = configV3.network.wsMaxPayload;
            cli_ux_1.default.action.stop();
        }
        if ((_o = configV3 === null || configV3 === void 0 ? void 0 : configV3.network) === null || _o === void 0 ? void 0 : _o.advertiseAddress) {
            cli_ux_1.default.action.start(`Setting config property 'network.advertiseAddress' to: ${configV3.network.advertiseAddress}.`);
            configV4.network.advertiseAddress = configV3.network.advertiseAddress;
            cli_ux_1.default.action.stop();
        }
    }
    cli_ux_1.default.action.start("Calculating and updating config property 'genesis.minimumCertifyHeight'.");
    configV4.genesis.minimumCertifyHeight =
        snapshotHeight +
            1 +
            (constants_1.POS_INIT_ROUNDS + constants_1.NUMBER_ACTIVE_VALIDATORS - 1) *
                (constants_1.NUMBER_ACTIVE_VALIDATORS + constants_1.NUMBER_STANDBY_VALIDATORS);
    cli_ux_1.default.action.stop();
    if (((_p = configV4.modules) === null || _p === void 0 ? void 0 : _p.pos) && !((_r = (_q = configV4.modules) === null || _q === void 0 ? void 0 : _q.pos) === null || _r === void 0 ? void 0 : _r.maxBFTWeightCap)) {
        cli_ux_1.default.action.start(`Setting config property 'modules.pos.maxBFTWeightCap' to: ${constants_1.MAX_BFT_WEIGHT_CAP}.`);
        configV4.modules.pos.maxBFTWeightCap = constants_1.MAX_BFT_WEIGHT_CAP;
        cli_ux_1.default.action.stop();
    }
    cli_ux_1.default.action.stop();
    return configV4;
};
exports.migrateUserConfig = migrateUserConfig;
const validateConfig = async (config) => {
    try {
        const mergedConfig = lisk_utils_1.objects.mergeDeep({}, lisk_framework_1.applicationConfigSchema.default, config);
        (await lisk_validator_1.validator.validate(lisk_framework_1.applicationConfigSchema, mergedConfig));
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.validateConfig = validateConfig;
const writeConfig = async (config, outputDir) => {
    if (!(0, fs_1.existsSync)(outputDir)) {
        (0, fs_1.mkdirSync)(outputDir, { recursive: true });
    }
    (0, fs_1.writeFileSync)((0, path_1.resolve)(outputDir, 'config.json'), JSON.stringify(config, null, '\t'));
};
exports.writeConfig = writeConfig;
//# sourceMappingURL=config.js.map